@@ -1,6 +1,6 @@
 /*
  * HEIF codec.
- * Copyright (c) 2017 struktur AG, Dirk Farin <farin@struktur.de>
+ * Copyright (c) 2017 Dirk Farin <dirk.farin@gmail.com>
  *
  * This file is part of libheif.
  *
@@ -40,6 +40,11 @@
 #include "hdr_sdr.h"
 #include "chroma_sampling.h"
 
+#if ENABLE_MULTITHREADING_SUPPORT
+
+#include <mutex>
+
+#endif
 
 #define DEBUG_ME 0
 #define DEBUG_PIPELINE_CREATION 0
@@ -140,10 +145,24 @@ static void __attribute__ ((unused)) print_spec(std::ostream& ostr, const std::s
 
 bool ColorState::operator==(const ColorState& b) const
 {
-  return (colorspace == b.colorspace &&
-          chroma == b.chroma &&
-          has_alpha == b.has_alpha &&
-          bits_per_pixel == b.bits_per_pixel);
+  bool mainParamsMatch = (colorspace == b.colorspace &&
+                          chroma == b.chroma &&
+                          has_alpha == b.has_alpha &&
+                          bits_per_pixel == b.bits_per_pixel);
+
+  if (!mainParamsMatch) {
+    return false;
+  }
+
+  if (colorspace == heif_colorspace_YCbCr) {
+    bool ycbcr_parameters_match = (nclx_profile.get_full_range_flag() == b.nclx_profile.get_full_range_flag() &&
+                                   nclx_profile.get_matrix_coefficients() == b.nclx_profile.get_matrix_coefficients() &&
+                                   nclx_profile.get_colour_primaries() == b.nclx_profile.get_colour_primaries());
+    return ycbcr_parameters_match;
+  }
+  else {
+    return true;
+  }
 }
 
 
@@ -151,70 +170,90 @@ struct Node
 {
   Node() = default;
 
-  Node(int prev, const ColorConversionOperation* _op, const ColorStateWithCost& state)
+  Node(int prev,
+       const std::shared_ptr<ColorConversionOperation>& _op,
+      //const ColorState& _input_state,
+       const ColorState& _output_state,
+       int _speed_cost)
   {
     prev_processed_idx = prev;
     op = _op;
-    color_state = state;
+    //input_state = _input_state;
+    output_state = _output_state;
+    speed_costs = _speed_cost;
   }
 
   int prev_processed_idx = -1;
-  const ColorConversionOperation* op;
-  ColorStateWithCost color_state;
+  std::shared_ptr<ColorConversionOperation> op;
+  //ColorState input_state;
+  ColorState output_state;
+  int speed_costs;
 };
 
-std::ostream& operator<<(std::ostream& ostr, const ColorState& state) {
-  return ostr << "colorspace=" << state.colorspace << " chroma=" << state.chroma
-              << " bpp(R)=" << state.bits_per_pixel
-              << " alpha=" << (state.has_alpha ? "yes" : "no")
-              << " nclx=" << (state.nclx_profile ? "yes" : "no");
+std::ostream& operator<<(std::ostream& ostr, const ColorState& state)
+{
+  ostr << "colorspace=" << state.colorspace << " chroma=" << state.chroma
+           << " bpp(R)=" << state.bits_per_pixel
+              << " alpha=" << (state.has_alpha ? "yes" : "no");
+
+  if (state.colorspace == heif_colorspace_YCbCr) {
+    ostr << " matrix-coefficients=" << state.nclx_profile.get_matrix_coefficients()
+         << " colour-primaries=" << state.nclx_profile.get_colour_primaries()
+         << " transfer-characteristics=" << state.nclx_profile.get_transfer_characteristics()
+         << " full-range=" << (state.nclx_profile.get_full_range_flag() ? "yes" : "no");
+  }
+
+  return ostr;
 }
 
-std::vector<ColorConversionOperation*> ColorConversionPipeline::m_operation_pool;
-
+std::vector<std::shared_ptr<ColorConversionOperation>> ColorConversionPipeline::m_operation_pool;
 
 void ColorConversionPipeline::init_ops()
 {
+#if ENABLE_MULTITHREADING_SUPPORT
+  static std::mutex init_ops_mutex;
+  std::lock_guard<std::mutex> lock(init_ops_mutex);
+#endif
   if (!m_operation_pool.empty()) {
     return;
   }
 
-  std::vector<ColorConversionOperation*>& ops = m_operation_pool;
-  ops.push_back(new Op_RGB_to_RGB24_32());
-  ops.push_back(new Op_RGB24_32_to_RGB());
-  ops.push_back(new Op_YCbCr_to_RGB<uint16_t>());
-  ops.push_back(new Op_YCbCr_to_RGB<uint8_t>());
-  ops.push_back(new Op_YCbCr420_to_RGB24());
-  ops.push_back(new Op_YCbCr420_to_RGB32());
-  ops.push_back(new Op_YCbCr420_to_RRGGBBaa());
-  ops.push_back(new Op_RGB_HDR_to_RRGGBBaa_BE());
-  ops.push_back(new Op_RGB_to_RRGGBBaa_BE());
-  ops.push_back(new Op_mono_to_YCbCr420());
-  ops.push_back(new Op_mono_to_RGB24_32());
-  ops.push_back(new Op_RRGGBBaa_swap_endianness());
-  ops.push_back(new Op_RRGGBBaa_BE_to_RGB_HDR());
-  ops.push_back(new Op_RGB24_32_to_YCbCr());
-  ops.push_back(new Op_RGB_to_YCbCr<uint8_t>());
-  ops.push_back(new Op_RGB_to_YCbCr<uint16_t>());
-  ops.push_back(new Op_RRGGBBxx_HDR_to_YCbCr420());
-  ops.push_back(new Op_RGB24_32_to_YCbCr444_GBR());
-  ops.push_back(new Op_drop_alpha_plane());
-  ops.push_back(new Op_to_hdr_planes());
-  ops.push_back(new Op_to_sdr_planes());
-  ops.push_back(new Op_YCbCr420_bilinear_to_YCbCr444<uint8_t>());
-  ops.push_back(new Op_YCbCr420_bilinear_to_YCbCr444<uint16_t>());
-  ops.push_back(new Op_YCbCr444_to_YCbCr420_average<uint8_t>());
-  ops.push_back(new Op_YCbCr444_to_YCbCr420_average<uint16_t>());
-  ops.push_back(new Op_Any_RGB_to_YCbCr_420_Sharp());
+  std::vector<std::shared_ptr<ColorConversionOperation>>& ops = m_operation_pool;
+  ops.push_back(std::make_shared<Op_RGB_to_RGB24_32>());
+  ops.push_back(std::make_shared<Op_RGB24_32_to_RGB>());
+  ops.push_back(std::make_shared<Op_YCbCr_to_RGB<uint16_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr_to_RGB<uint8_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr420_to_RGB24>());
+  ops.push_back(std::make_shared<Op_YCbCr420_to_RGB32>());
+  ops.push_back(std::make_shared<Op_YCbCr420_to_RRGGBBaa>());
+  ops.push_back(std::make_shared<Op_RGB_HDR_to_RRGGBBaa_BE>());
+  ops.push_back(std::make_shared<Op_RGB_to_RRGGBBaa_BE>());
+  ops.push_back(std::make_shared<Op_mono_to_YCbCr420>());
+  ops.push_back(std::make_shared<Op_mono_to_RGB24_32>());
+  ops.push_back(std::make_shared<Op_RRGGBBaa_swap_endianness>());
+  ops.push_back(std::make_shared<Op_RRGGBBaa_BE_to_RGB_HDR>());
+  ops.push_back(std::make_shared<Op_RGB24_32_to_YCbCr>());
+  ops.push_back(std::make_shared<Op_RGB_to_YCbCr<uint8_t>>());
+  ops.push_back(std::make_shared<Op_RGB_to_YCbCr<uint16_t>>());
+  ops.push_back(std::make_shared<Op_RRGGBBxx_HDR_to_YCbCr420>());
+  ops.push_back(std::make_shared<Op_RGB24_32_to_YCbCr444_GBR>());
+  ops.push_back(std::make_shared<Op_drop_alpha_plane>());
+  ops.push_back(std::make_shared<Op_to_hdr_planes>());
+  ops.push_back(std::make_shared<Op_to_sdr_planes>());
+  ops.push_back(std::make_shared<Op_YCbCr420_bilinear_to_YCbCr444<uint8_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr420_bilinear_to_YCbCr444<uint16_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr422_bilinear_to_YCbCr444<uint8_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr422_bilinear_to_YCbCr444<uint16_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr444_to_YCbCr420_average<uint8_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr444_to_YCbCr420_average<uint16_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr444_to_YCbCr422_average<uint8_t>>());
+  ops.push_back(std::make_shared<Op_YCbCr444_to_YCbCr422_average<uint16_t>>());
+  ops.push_back(std::make_shared<Op_Any_RGB_to_YCbCr_420_Sharp>());
 }
 
 
 void ColorConversionPipeline::release_ops()
 {
-  for (auto& op : m_operation_pool) {
-    delete op;
-  }
-
   m_operation_pool.clear();
 }
 
@@ -238,19 +277,19 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
 
   init_ops(); // to be sure these are initialized even without heif_init()
 
-  std::vector<ColorConversionOperation*>& ops = m_operation_pool;
+  std::vector<std::shared_ptr<ColorConversionOperation>>& ops = m_operation_pool;
 
   // --- Dijkstra search for the minimum-cost conversion pipeline
 
   std::vector<Node> processed_states;
   std::vector<Node> border_states;
-  border_states.push_back({-1, nullptr, {input_state, 0}});
+  border_states.push_back({-1, nullptr, input_state, 0});
 
   while (!border_states.empty()) {
     int minIdx = -1;
     int minCost = std::numeric_limits<int>::max();
-    for (int i = 0; i < (int)border_states.size(); i++) {
-      int cost = border_states[i].color_state.speed_costs;
+    for (int i = 0; i < (int) border_states.size(); i++) {
+      int cost = border_states[i].speed_costs;
       if (cost < minCost) {
         minIdx = i;
         minCost = cost;
@@ -268,11 +307,11 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
     border_states.pop_back();
 
 #if DEBUG_PIPELINE_CREATION
-    std::cerr << "- expand node: " << processed_states.back().color_state.color_state
-        << " with cost " << processed_states.back().color_state.speed_costs << " \n";
+    std::cerr << "- expand node: " << processed_states.back().output_state
+        << " with cost " << processed_states.back().speed_costs << " \n";
 #endif
 
-    if (processed_states.back().color_state.color_state == target_state) {
+    if (processed_states.back().output_state == target_state) {
       // end-state found, backtrack path to find conversion pipeline
 
       size_t idx = processed_states.size() - 1;
@@ -288,13 +327,18 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
       int step = 0;
       while (idx > 0) {
         m_conversion_steps[len - 1 - step].operation = processed_states[idx].op;
-        m_conversion_steps[len - 1 - step].output_state = processed_states[idx].color_state.color_state;
+        m_conversion_steps[len - 1 - step].output_state = processed_states[idx].output_state;
+        if (step > 0) {
+          m_conversion_steps[len - step].input_state = m_conversion_steps[len - 1 - step].output_state;
+        }
 
         //printf("cost: %f\n",processed_states[idx].color_state.costs.total(options.criterion));
         idx = processed_states[idx].prev_processed_idx;
         step++;
       }
 
+      m_conversion_steps[0].input_state = input_state;
+
       assert(m_conversion_steps.back().output_state == target_state);
 
 #if DEBUG_ME
@@ -314,18 +358,18 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
       std::cerr << "-- apply op: " << typeid(op).name() << "\n";
 #endif
 
-      auto out_states = op_ptr->state_after_conversion(processed_states.back().color_state.color_state,
+      auto out_states = op_ptr->state_after_conversion(processed_states.back().output_state,
                                                        target_state,
                                                        options);
       for (const auto& out_state : out_states) {
-        int new_op_costs = out_state.speed_costs + processed_states.back().color_state.speed_costs;
+        int new_op_costs = out_state.speed_costs + processed_states.back().speed_costs;
 #if DEBUG_PIPELINE_CREATION
         std::cerr << "--- " << out_state.color_state << " with cost " << new_op_costs << "\n";
 #endif
 
         bool state_exists = false;
         for (const auto& s : processed_states) {
-          if (s.color_state.color_state == out_state.color_state) {
+          if (s.output_state == out_state.color_state) {
             state_exists = true;
             break;
           }
@@ -333,17 +377,18 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
 
         if (!state_exists) {
           for (auto& s : border_states) {
-            if (s.color_state.color_state == out_state.color_state) {
+            if (s.output_state == out_state.color_state) {
               state_exists = true;
 
               // if we reached the same border node with a lower cost, replace the border node
 
-              if (s.color_state.speed_costs > new_op_costs) {
+              if (s.speed_costs > new_op_costs) {
                 s = {(int) (processed_states.size() - 1),
                      op_ptr,
-                     out_state};
+                     out_state.color_state,
+                     out_state.speed_costs};
 
-                s.color_state.speed_costs = new_op_costs;
+                s.speed_costs = new_op_costs;
               }
               break;
             }
@@ -355,9 +400,12 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
 
         if (!state_exists) {
           ColorStateWithCost s = out_state;
-          s.speed_costs = s.speed_costs + processed_states.back().color_state.speed_costs;
+          s.speed_costs = s.speed_costs + processed_states.back().speed_costs;
 
-          border_states.push_back({(int) (processed_states.size() - 1), op_ptr, s});
+          border_states.push_back({(int) (processed_states.size() - 1),
+                                   op_ptr,
+                                   s.color_state,
+                                   s.speed_costs});
         }
       }
     }
@@ -391,14 +439,16 @@ std::shared_ptr<HeifPixelImage> ColorConversionPipeline::convert_image(const std
     print_spec(std::cerr, in);
 #endif
 
-    out = step.operation->convert_colorspace(in, step.output_state, m_options);
+    out = step.operation->convert_colorspace(in, step.input_state, step.output_state, m_options);
     if (!out) {
       return nullptr; // TODO: we should return a proper error
     }
 
     // --- pass the color profiles to the new image
 
-    out->set_color_profile_nclx(step.output_state.nclx_profile);
+    auto output_nclx = std::make_shared<color_profile_nclx>();
+    *output_nclx = step.output_state.nclx_profile;
+    out->set_color_profile_nclx(output_nclx);
     out->set_color_profile_icc(in->get_color_profile_icc());
 
     out->set_premultiplied_alpha(in->is_premultiplied_alpha());
@@ -467,7 +517,23 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
   input_state.colorspace = input->get_colorspace();
   input_state.chroma = input->get_chroma_format();
   input_state.has_alpha = input->has_channel(heif_channel_Alpha) || is_chroma_with_alpha(input->get_chroma_format());
-  input_state.nclx_profile = input->get_color_profile_nclx();
+  if (input->get_color_profile_nclx()) {
+    input_state.nclx_profile = *input->get_color_profile_nclx();
+  }
+
+  // If some input nclx values are unspecified, use CCIR-601 values as default.
+
+  if (input_state.nclx_profile.get_matrix_coefficients() == heif_matrix_coefficients_unspecified) {
+    input_state.nclx_profile.set_matrix_coefficients(heif_matrix_coefficients_ITU_R_BT_601_6);
+  }
+
+  if (input_state.nclx_profile.get_colour_primaries() == heif_color_primaries_unspecified) {
+    input_state.nclx_profile.set_colour_primaries(heif_color_primaries_ITU_R_BT_601_6);
+  }
+
+  if (input_state.nclx_profile.get_transfer_characteristics() == heif_color_primaries_unspecified) {
+    input_state.nclx_profile.set_transfer_characteristics(heif_transfer_characteristic_ITU_R_BT_601_6);
+  }
 
   std::set<enum heif_channel> channels = input->get_channel_set();
   assert(!channels.empty());
@@ -476,7 +542,23 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
   ColorState output_state = input_state;
   output_state.colorspace = target_colorspace;
   output_state.chroma = target_chroma;
-  output_state.nclx_profile = target_profile;
+  if (target_profile) {
+    output_state.nclx_profile = *target_profile;
+  }
+
+  // If some output nclx values are unspecified, set the to the same as the input.
+
+  if (output_state.nclx_profile.get_matrix_coefficients() == heif_matrix_coefficients_unspecified) {
+    output_state.nclx_profile.set_matrix_coefficients(input_state.nclx_profile.get_matrix_coefficients());
+  }
+
+  if (output_state.nclx_profile.get_colour_primaries() == heif_color_primaries_unspecified) {
+    output_state.nclx_profile.set_colour_primaries(input_state.nclx_profile.get_colour_primaries());
+  }
+
+  if (output_state.nclx_profile.get_transfer_characteristics() == heif_color_primaries_unspecified) {
+    output_state.nclx_profile.set_transfer_characteristics(input_state.nclx_profile.get_transfer_characteristics());
+  }
 
   // If we convert to an interleaved format, we want alpha only if present in the
   // interleaved output format.
