@@ -253,12 +253,21 @@ Error HeifFile::parse_heif_file(BitstreamRange& range)
     std::shared_ptr<Box> box;
     Error error = Box::read(range, &box);
 
+    if (range.error() || range.eof()) {
+      break;
+    }
+
     // When an EOF error is returned, this is not really a fatal exception,
     // but simply the indication that we reached the end of the file.
-    if (error != Error::Ok || range.error() || range.eof()) {
+    // TODO: this design should be cleaned up
+    if (error.error_code == heif_error_Invalid_input && error.sub_error_code == heif_suberror_End_of_data) {
       break;
     }
 
+    if (error != Error::Ok) {
+      return error;
+    }
+
     m_top_level_boxes.push_back(box);
 
 
